@@ -225,7 +225,7 @@ int inet_ntoa_lock = 0;
 
 static bool php_open_listen_sock(php_socket *sock, int port, int backlog) /* {{{ */
 {
-	struct sockaddr_in  la;
+	struct sockaddr_in  la = {0};
 	struct hostent		*hp;
 
 #ifndef PHP_WIN32
@@ -921,7 +921,7 @@ PHP_FUNCTION(socket_read)
 PHP_FUNCTION(socket_getsockname)
 {
 	zval					*arg1, *addr, *port = NULL;
-	php_sockaddr_storage	sa_storage;
+	php_sockaddr_storage	sa_storage = {0};
 	php_socket				*php_sock;
 	struct sockaddr			*sa;
 	struct sockaddr_in		*sin;
@@ -998,7 +998,7 @@ PHP_FUNCTION(socket_getsockname)
 PHP_FUNCTION(socket_getpeername)
 {
 	zval					*arg1, *arg2, *arg3 = NULL;
-	php_sockaddr_storage	sa_storage;
+	php_sockaddr_storage	sa_storage = {0};
 	php_socket				*php_sock;
 	struct sockaddr			*sa;
 	struct sockaddr_in		*sin;
