@@ -1,13 +1,11 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2015 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -25,7 +23,7 @@
 #include "phpdbg_prompt.h"
 #include "phpdbg_io.h"
 
-ZEND_EXTERN_MODULE_GLOBALS(phpdbg);
+ZEND_EXTERN_MODULE_GLOBALS(phpdbg)
 
 static inline const char *phpdbg_command_name(const phpdbg_command_t *command, char *buffer) {
 	size_t pos = 0;
@@ -103,7 +101,7 @@ PHPDBG_API char* phpdbg_param_tostring(const phpdbg_param_t *param, char **point
 		break;
 
 		case NUMERIC_PARAM:
-			ZEND_IGNORE_VALUE(asprintf(pointer, "%li", param->num));
+			ZEND_IGNORE_VALUE(asprintf(pointer, ZEND_LONG_FMT, param->num));
 		break;
 
 		case METHOD_PARAM:
@@ -112,18 +110,18 @@ PHPDBG_API char* phpdbg_param_tostring(const phpdbg_param_t *param, char **point
 
 		case FILE_PARAM:
 			if (param->num) {
-				ZEND_IGNORE_VALUE(asprintf(pointer, "%s:%lu#%lu", param->file.name, param->file.line, param->num));
+				ZEND_IGNORE_VALUE(asprintf(pointer, "%s:"ZEND_ULONG_FMT"#"ZEND_ULONG_FMT, param->file.name, param->file.line, param->num));
 			} else {
-				ZEND_IGNORE_VALUE(asprintf(pointer, "%s:%lu", param->file.name, param->file.line));
+				ZEND_IGNORE_VALUE(asprintf(pointer, "%s:"ZEND_ULONG_FMT, param->file.name, param->file.line));
 			}
 		break;
 
 		case NUMERIC_FUNCTION_PARAM:
-			ZEND_IGNORE_VALUE(asprintf(pointer, "%s#%lu", param->str, param->num));
+			ZEND_IGNORE_VALUE(asprintf(pointer, "%s#"ZEND_ULONG_FMT, param->str, param->num));
 		break;
 
 		case NUMERIC_METHOD_PARAM:
-			ZEND_IGNORE_VALUE(asprintf(pointer, "%s::%s#%lu", param->method.class, param->method.name, param->num));
+			ZEND_IGNORE_VALUE(asprintf(pointer, "%s::%s#"ZEND_ULONG_FMT, param->method.class, param->method.name, param->num));
 		break;
 
 		default:
@@ -201,16 +199,16 @@ PHPDBG_API zend_ulong phpdbg_hash_param(const phpdbg_param_t *param) /* {{{ */
 		break;
 
 		case STR_PARAM:
-			hash += zend_inline_hash_func(param->str, param->len);
+			hash += zend_hash_func(param->str, param->len);
 		break;
 
 		case METHOD_PARAM:
-			hash += zend_inline_hash_func(param->method.class, strlen(param->method.class));
-			hash += zend_inline_hash_func(param->method.name, strlen(param->method.name));
+			hash += zend_hash_func(param->method.class, strlen(param->method.class));
+			hash += zend_hash_func(param->method.name, strlen(param->method.name));
 		break;
 
 		case FILE_PARAM:
-			hash += zend_inline_hash_func(param->file.name, strlen(param->file.name));
+			hash += zend_hash_func(param->file.name, strlen(param->file.name));
 			hash += param->file.line;
 			if (param->num)
 				hash += param->num;
@@ -225,13 +223,13 @@ PHPDBG_API zend_ulong phpdbg_hash_param(const phpdbg_param_t *param) /* {{{ */
 		break;
 
 		case NUMERIC_FUNCTION_PARAM:
-			hash += zend_inline_hash_func(param->str, param->len);
+			hash += zend_hash_func(param->str, param->len);
 			hash += param->num;
 		break;
 
 		case NUMERIC_METHOD_PARAM:
-			hash += zend_inline_hash_func(param->method.class, strlen(param->method.class));
-			hash += zend_inline_hash_func(param->method.name, strlen(param->method.name));
+			hash += zend_hash_func(param->method.class, strlen(param->method.class));
+			hash += zend_hash_func(param->method.name, strlen(param->method.name));
 			if (param->num)
 				hash+= param->num;
 		break;
@@ -246,7 +244,7 @@ PHPDBG_API zend_ulong phpdbg_hash_param(const phpdbg_param_t *param) /* {{{ */
 	return hash;
 } /* }}} */
 
-PHPDBG_API zend_bool phpdbg_match_param(const phpdbg_param_t *l, const phpdbg_param_t *r) /* {{{ */
+PHPDBG_API bool phpdbg_match_param(const phpdbg_param_t *l, const phpdbg_param_t *r) /* {{{ */
 {
 	if (l && r) {
 		if (l->type == r->type) {
@@ -260,7 +258,7 @@ PHPDBG_API zend_bool phpdbg_match_param(const phpdbg_param_t *l, const phpdbg_pa
 					if (l->num != r->num) {
 						break;
 					}
-				/* break intentionally omitted */
+				ZEND_FALLTHROUGH;
 
 				case STR_PARAM:
 					return (l->len == r->len) &&
@@ -290,7 +288,7 @@ PHPDBG_API zend_bool phpdbg_match_param(const phpdbg_param_t *l, const phpdbg_pa
 					if (l->num != r->num) {
 						break;
 					}
-				/* break intentionally omitted */
+				ZEND_FALLTHROUGH;
 
 				case METHOD_PARAM: {
 					size_t lengths[2] = {
@@ -333,11 +331,11 @@ PHPDBG_API void phpdbg_param_debug(const phpdbg_param_t *param, const char *msg)
 			break;
 
 			case NUMERIC_FILE_PARAM:
-				fprintf(stderr, "%s NUMERIC_FILE_PARAM(%s:#%lu)\n", msg, param->file.name, param->file.line);
+				fprintf(stderr, "%s NUMERIC_FILE_PARAM(%s:#"ZEND_ULONG_FMT")\n", msg, param->file.name, param->file.line);
 			break;
 
 			case FILE_PARAM:
-				fprintf(stderr, "%s FILE_PARAM(%s:%lu)\n", msg, param->file.name, param->file.line);
+				fprintf(stderr, "%s FILE_PARAM(%s:"ZEND_ULONG_FMT")\n", msg, param->file.name, param->file.line);
 			break;
 
 			case METHOD_PARAM:
@@ -349,11 +347,11 @@ PHPDBG_API void phpdbg_param_debug(const phpdbg_param_t *param, const char *msg)
 			break;
 
 			case NUMERIC_FUNCTION_PARAM:
-				fprintf(stderr, "%s NUMERIC_FUNCTION_PARAM(%s::%ld)\n", msg, param->str, param->num);
+				fprintf(stderr, "%s NUMERIC_FUNCTION_PARAM(%s::"ZEND_LONG_FMT")\n", msg, param->str, param->num);
 			break;
 
 			case NUMERIC_PARAM:
-				fprintf(stderr, "%s NUMERIC_PARAM(%ld)\n", msg, param->num);
+				fprintf(stderr, "%s NUMERIC_PARAM("ZEND_LONG_FMT")\n", msg, param->num);
 			break;
 
 			case COND_PARAM:
@@ -373,7 +371,9 @@ PHPDBG_API void phpdbg_param_debug(const phpdbg_param_t *param, const char *msg)
 
 /* {{{ */
 PHPDBG_API void phpdbg_stack_free(phpdbg_param_t *stack) {
-	if (stack && stack->next) {
+	ZEND_ASSERT(stack != NULL);
+
+	if (stack->next) {
 		phpdbg_param_t *remove = stack->next;
 
 		while (remove) {
@@ -424,10 +424,9 @@ PHPDBG_API void phpdbg_stack_free(phpdbg_param_t *stack) {
 				remove = next;
 			else break;
 		}
+
+		stack->next = NULL;
 	}
-
-
-	stack->next = NULL;
 } /* }}} */
 
 /* {{{ */
@@ -470,10 +469,10 @@ PHPDBG_API int phpdbg_stack_verify(const phpdbg_command_t *command, phpdbg_param
 		char buffer[128] = {0,};
 		const phpdbg_param_t *top = (stack != NULL) ? *stack : NULL;
 		const char *arg = command->args;
-		size_t least = 0L,
-		       received = 0L,
-		       current = 0L;
-		zend_bool optional = 0;
+		zend_ulong least = 0L,
+		           received = 0L,
+		           current = 0L;
+		bool optional = 0;
 
 		/* check for arg spec */
 		if (!(arg) || !(*arg)) {
@@ -481,7 +480,7 @@ PHPDBG_API int phpdbg_stack_verify(const phpdbg_command_t *command, phpdbg_param
 				return SUCCESS;
 			}
 
-			phpdbg_error("command", "type=\"toomanyargs\" command=\"%s\" expected=\"0\"", "The command \"%s\" expected no arguments",
+			phpdbg_error("The command \"%s\" expected no arguments",
 				phpdbg_command_name(command, buffer));
 			return FAILURE;
 		}
@@ -501,14 +500,14 @@ PHPDBG_API int phpdbg_stack_verify(const phpdbg_command_t *command, phpdbg_param
 
 #define verify_arg(e, a, t) if (!(a)) { \
 	if (!optional) { \
-		phpdbg_error("command", "type=\"noarg\" command=\"%s\" expected=\"%s\" num=\"%lu\"", "The command \"%s\" expected %s and got nothing at parameter %lu", \
+		phpdbg_error("The command \"%s\" expected %s and got nothing at parameter "ZEND_ULONG_FMT, \
 			phpdbg_command_name(command, buffer), \
 			(e), \
 			current); \
 		return FAILURE;\
 	} \
 } else if ((a)->type != (t)) { \
-	phpdbg_error("command", "type=\"wrongarg\" command=\"%s\" expected=\"%s\" got=\"%s\" num=\"%lu\"", "The command \"%s\" expected %s and got %s at parameter %lu", \
+	phpdbg_error("The command \"%s\" expected %s and got %s at parameter "ZEND_ULONG_FMT, \
 		phpdbg_command_name(command, buffer), \
 		(e),\
 		phpdbg_get_param_type((a)), \
@@ -556,7 +555,7 @@ PHPDBG_API int phpdbg_stack_verify(const phpdbg_command_t *command, phpdbg_param
 #undef verify_arg
 
 		if ((received < least)) {
-			phpdbg_error("command", "type=\"toofewargs\" command=\"%s\" expected=\"%d\" argtypes=\"%s\" got=\"%d\"", "The command \"%s\" expected at least %lu arguments (%s) and received %lu",
+			phpdbg_error("The command \"%s\" expected at least "ZEND_ULONG_FMT" arguments (%s) and received "ZEND_ULONG_FMT,
 				phpdbg_command_name(command, buffer),
 				least,
 				command->args,
@@ -573,7 +572,7 @@ PHPDBG_API const phpdbg_command_t *phpdbg_stack_resolve(const phpdbg_command_t *
 	const phpdbg_command_t *command = commands;
 	phpdbg_param_t *name = *top;
 	const phpdbg_command_t *matched[3] = {NULL, NULL, NULL};
-	ulong matches = 0L;
+	zend_ulong matches = 0L;
 
 	while (command && command->name && command->handler) {
 		if (name->len == 1 || command->name_len >= name->len) {
@@ -610,9 +609,9 @@ PHPDBG_API const phpdbg_command_t *phpdbg_stack_resolve(const phpdbg_command_t *
 	switch (matches) {
 		case 0:
 			if (parent) {
-				phpdbg_error("command", "type=\"notfound\" command=\"%s\" subcommand=\"%s\"", "The command \"%s %s\" could not be found", parent->name, name->str);
+				phpdbg_error("The command \"%s %s\" could not be found", parent->name, name->str);
 			} else {
-				phpdbg_error("command", "type=\"notfound\" command=\"%s\"", "The command \"%s\" could not be found", name->str);
+				phpdbg_error("The command \"%s\" could not be found", name->str);
 			}
 			return parent;
 
@@ -645,7 +644,7 @@ PHPDBG_API const phpdbg_command_t *phpdbg_stack_resolve(const phpdbg_command_t *
 			}
 
 			/* ", " separated matches */
-			phpdbg_error("command", "type=\"ambiguous\" command=\"%s\" matches=\"%lu\" matched=\"%s\"", "The command \"%s\" is ambigious, matching %lu commands (%s)", name->str, matches, list);
+			phpdbg_error("The command \"%s\" is ambiguous, matching "ZEND_ULONG_FMT" commands (%s)", name->str, matches, list);
 			efree(list);
 
 			return NULL;
@@ -661,7 +660,7 @@ PHPDBG_API const phpdbg_command_t *phpdbg_stack_resolve(const phpdbg_command_t *
 	return NULL;
 } /* }}} */
 
-static int phpdbg_internal_stack_execute(phpdbg_param_t *stack, zend_bool allow_async_unsafe) {
+static int phpdbg_internal_stack_execute(phpdbg_param_t *stack, bool allow_async_unsafe) {
 	const phpdbg_command_t *handler = NULL;
 	phpdbg_param_t *top = (phpdbg_param_t *) stack->next;
 
@@ -673,7 +672,7 @@ static int phpdbg_internal_stack_execute(phpdbg_param_t *stack, zend_bool allow_
 
 		case RUN_PARAM:
 			if (!allow_async_unsafe) {
-				phpdbg_error("signalsegv", "command=\"run\"", "run command is disallowed during hard interrupt");
+				phpdbg_error("run command is disallowed during hard interrupt");
 			}
 			phpdbg_activate_err_buf(0);
 			phpdbg_free_err_buf();
@@ -681,7 +680,7 @@ static int phpdbg_internal_stack_execute(phpdbg_param_t *stack, zend_bool allow_
 
 		case SHELL_PARAM:
 			if (!allow_async_unsafe) {
-				phpdbg_error("signalsegv", "command=\"sh\"", "sh command is disallowed during hard interrupt");
+				phpdbg_error("sh command is disallowed during hard interrupt");
 				return FAILURE;
 			}
 			phpdbg_activate_err_buf(0);
@@ -693,7 +692,7 @@ static int phpdbg_internal_stack_execute(phpdbg_param_t *stack, zend_bool allow_
 
 			if (handler) {
 				if (!allow_async_unsafe && !(handler->flags & PHPDBG_ASYNC_SAFE)) {
-					phpdbg_error("signalsegv", "command=\"%s\"", "%s command is disallowed during hard interrupt", handler->name);
+					phpdbg_error("%s command is disallowed during hard interrupt", handler->name);
 					return FAILURE;
 				}
 
@@ -706,7 +705,7 @@ static int phpdbg_internal_stack_execute(phpdbg_param_t *stack, zend_bool allow_
 		} return FAILURE;
 
 		default:
-			phpdbg_error("command", "type=\"invalidcommand\"", "The first parameter makes no sense !");
+			phpdbg_error("The first parameter makes no sense !");
 			return FAILURE;
 	}
 
@@ -714,16 +713,16 @@ static int phpdbg_internal_stack_execute(phpdbg_param_t *stack, zend_bool allow_
 } /* }}} */
 
 /* {{{ */
-PHPDBG_API int phpdbg_stack_execute(phpdbg_param_t *stack, zend_bool allow_async_unsafe) {
+PHPDBG_API int phpdbg_stack_execute(phpdbg_param_t *stack, bool allow_async_unsafe) {
 	phpdbg_param_t *top = stack;
 
 	if (stack->type != STACK_PARAM) {
-		phpdbg_error("command", "type=\"nostack\"", "The passed argument was not a stack !");
+		phpdbg_error("The passed argument was not a stack !");
 		return FAILURE;
 	}
 
 	if (!stack->len) {
-		phpdbg_error("command", "type=\"emptystack\"", "The stack contains nothing !");
+		phpdbg_error("The stack contains nothing !");
 		return FAILURE;
 	}
 
@@ -739,51 +738,33 @@ PHPDBG_API int phpdbg_stack_execute(phpdbg_param_t *stack, zend_bool allow_async
 	return SUCCESS;
 } /* }}} */
 
-PHPDBG_API char *phpdbg_read_input(char *buffered) /* {{{ */
+PHPDBG_API char *phpdbg_read_input(const char *buffered) /* {{{ */
 {
 	char buf[PHPDBG_MAX_CMD];
-	char *cmd = NULL;
 	char *buffer = NULL;
 
 	if ((PHPDBG_G(flags) & (PHPDBG_IS_STOPPING | PHPDBG_IS_RUNNING)) != PHPDBG_IS_STOPPING) {
-		if ((PHPDBG_G(flags) & PHPDBG_IS_REMOTE) && (buffered == NULL) && !phpdbg_active_sigsafe_mem()) {
-			fflush(PHPDBG_G(io)[PHPDBG_STDOUT].ptr);
-		}
-
 		if (buffered == NULL) {
-#define USE_LIB_STAR (defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDIT))
-			/* note: EOF makes readline write prompt again in local console mode - and ignored if compiled without readline */
-#if USE_LIB_STAR
-			if ((PHPDBG_G(flags) & PHPDBG_IS_REMOTE) || !isatty(PHPDBG_G(io)[PHPDBG_STDIN].fd))
-#endif
-			{
-				phpdbg_write("prompt", "", "%s", phpdbg_get_prompt());
-				phpdbg_consume_stdin_line(cmd = buf);
-			}
-#if USE_LIB_STAR
-			else {
-				cmd = readline(phpdbg_get_prompt());
-				PHPDBG_G(last_was_newline) = 1;
+#ifdef HAVE_PHPDBG_READLINE
+			char *cmd = readline(phpdbg_get_prompt());
+			PHPDBG_G(last_was_newline) = 1;
 
-				if (!cmd) {
-					PHPDBG_G(flags) |= PHPDBG_IS_QUITTING | PHPDBG_IS_DISCONNECTED;
-					zend_bailout();
-				}
-
-				add_history(cmd);
+			if (!cmd) {
+				PHPDBG_G(flags) |= PHPDBG_IS_QUITTING;
+				zend_bailout();
 			}
+
+			add_history(cmd);
+			buffer = estrdup(cmd);
+			free(cmd);
+#else
+			phpdbg_write("%s", phpdbg_get_prompt());
+			phpdbg_consume_stdin_line(buf);
+			buffer = estrdup(buf);
 #endif
 		} else {
-			cmd = buffered;
+			buffer = estrdup(buffered);
 		}
-
-		buffer = estrdup(cmd);
-
-#if USE_LIB_STAR
-		if (!buffered && cmd &&	!(PHPDBG_G(flags) & PHPDBG_IS_REMOTE) && isatty(PHPDBG_G(io)[PHPDBG_STDIN].fd)) {
-			free(cmd);
-		}
-#endif
 	}
 
 	if (buffer && isspace(*buffer)) {
@@ -817,21 +798,19 @@ PHPDBG_API void phpdbg_destroy_input(char **input) /*{{{ */
 } /* }}} */
 
 PHPDBG_API int phpdbg_ask_user_permission(const char *question) {
-	if (!(PHPDBG_G(flags) & PHPDBG_WRITE_XML)) {
-		char buf[PHPDBG_MAX_CMD];
-		phpdbg_out("%s", question);
-		phpdbg_out(" (type y or n): ");
+	char buf[PHPDBG_MAX_CMD];
+	phpdbg_out("%s", question);
+	phpdbg_out(" (type y or n): ");
 
-		while (1) {
-			phpdbg_consume_stdin_line(buf);
-			if (buf[1] == '\n' && (buf[0] == 'y' || buf[0] == 'n')) {
-				if (buf[0] == 'y') {
-					return SUCCESS;
-				}
-				return FAILURE;
+	while (1) {
+		phpdbg_consume_stdin_line(buf);
+		if ((buf[1] == '\n' || (buf[1] == '\r' && buf[2] == '\n')) && (buf[0] == 'y' || buf[0] == 'n')) {
+			if (buf[0] == 'y') {
+				return SUCCESS;
 			}
-			phpdbg_out("Please enter either y (yes) or n (no): ");
+			return FAILURE;
 		}
+		phpdbg_out("Please enter either y (yes) or n (no): ");
 	}
 
 	return SUCCESS;
